@@ -1,5 +1,5 @@
 /*
- * "$Id: main.c 7725 2008-07-14 06:16:34Z mike $"
+ * "$Id: main.c 7914 2008-09-07 21:58:01Z mike $"
  *
  *   Scheduler main loop for the Common UNIX Printing System (CUPS).
  *
@@ -633,6 +633,23 @@ main(int  argc,				/* I - Number of command-line args */
   cupsdStartSystemMonitor();
 #endif /* __APPLE__ */
 
+ /*
+  * Send server-started event...
+  */
+
+#ifdef HAVE_LAUNCHD
+  if (Launchd)
+    cupsdAddEvent(CUPSD_EVENT_SERVER_STARTED, NULL, NULL,
+                  "Scheduler started via launchd.");
+  else
+#endif /* HAVE_LAUNCHD */
+  if (fg)
+    cupsdAddEvent(CUPSD_EVENT_SERVER_STARTED, NULL, NULL,
+                  "Scheduler started in foreground.");
+  else
+    cupsdAddEvent(CUPSD_EVENT_SERVER_STARTED, NULL, NULL,
+                  "Scheduler started in background.");
+
  /*
   * Start any pending print jobs...
   */
@@ -741,6 +758,13 @@ main(int  argc,				/* I - Number of command-line args */
         */
 
         cupsdStartServer();
+
+       /*
+        * Send a server-restarted event...
+	*/
+
+        cupsdAddEvent(CUPSD_EVENT_SERVER_RESTARTED, NULL, NULL,
+                      "Scheduler restarted.");
       }
     }
 
@@ -1070,10 +1094,18 @@ main(int  argc,				/* I - Number of command-line args */
   */
 
   if (stop_scheduler)
+  {
     cupsdLogMessage(CUPSD_LOG_INFO, "Scheduler shutting down normally.");
+    cupsdAddEvent(CUPSD_EVENT_SERVER_STOPPED, NULL, NULL,
+                  "Scheduler shutting down normally.");
+  }
   else
+  {
     cupsdLogMessage(CUPSD_LOG_ERROR,
                     "Scheduler shutting down due to program error.");
+    cupsdAddEvent(CUPSD_EVENT_SERVER_STOPPED, NULL, NULL,
+                  "Scheduler shutting down due to program error.");
+  }
 
  /*
   * Close all network clients...
@@ -1981,5 +2013,5 @@ usage(int status)			/* O - Exit status */
 
 
 /*
- * End of "$Id: main.c 7725 2008-07-14 06:16:34Z mike $".
+ * End of "$Id: main.c 7914 2008-09-07 21:58:01Z mike $".
  */
